@@ -32,16 +32,20 @@
  * AN ACTION OF CONTRACT, NEGLIGENCE OR OTHER TORTIOUS ACTION, ARISING
  * OUT OF OR IN CONNECTION WITH THE USE OR PERFORMANCE OF THIS SOFTWARE.
  */
+#ifdef HAVE_CONFIG_H
+#include "config.h"
+#endif
 
 #include <stdio.h>
 #include <string.h>
 #include <sys/types.h>
 #include <sys/time.h>
 
-#include "pppd.h"
+#include "pppd-private.h"
 #include "cbcp.h"
 #include "fsm.h"
 #include "lcp.h"
+#include "options.h"
 
 
 /*
@@ -49,7 +53,7 @@
  */
 static int setcbcp (char **);
 
-static option_t cbcp_option_list[] = {
+static struct option cbcp_option_list[] = {
     { "callback", o_special, (void *)setcbcp,
       "Ask for callback", OPT_PRIO | OPT_A2STRVAL, &cbcp[0].us_number },
     { NULL }
@@ -458,6 +462,6 @@ static void
 cbcp_up(cbcp_state *us)
 {
     persist = 0;
-    status = EXIT_CALLBACK;
+    ppp_set_status(EXIT_CALLBACK);
     lcp_close(0, "Call me back, please");
 }
