@@ -34,7 +34,11 @@
  *
  */
 
-#include "pppd.h"
+#ifdef HAVE_CONFIG_H
+#include "config.h"
+#endif
+
+#include "pppd-private.h"
 
 
 /*
