@@ -40,23 +40,27 @@
  * OUT OF OR IN CONNECTION WITH THE USE OR PERFORMANCE OF THIS SOFTWARE.
  */
 
-#define RCSID	"$Id: lcp.c,v 1.76 2006/05/22 00:04:07 paulus Exp $"
-
-/*
- * TODO:
- */
+#ifdef HAVE_CONFIG_H
+#include "config.h"
+#endif
 
 #include <stdio.h>
 #include <string.h>
 #include <stdlib.h>
+#include <fcntl.h>
+#include <string.h>
+#include <time.h>
+#include <arpa/inet.h>
+#include <sys/mman.h>
 
-#include "pppd.h"
+#include "pppd-private.h"
+#include "options.h"
 #include "fsm.h"
 #include "lcp.h"
-#include "chap-new.h"
+#include "eap.h"
+#include "chap.h"
 #include "magic.h"
-
-static const char rcsid[] = RCSID;
+#include "multilink.h"
 
 /*
  * When the link comes up we want to be able to wait for a short while,
@@ -66,25 +70,35 @@ static const char rcsid[] = RCSID;
 /* steal a bit in fsm flags word */
 #define DELAYED_UP	0x100
 
-static void lcp_delayed_up __P((void *));
+static void lcp_delayed_up(void *);
+
+/*
+ * These definitions relate to the measurement and logging of round-trip
+ * time (RTT) of LCP echo-requests implemented in lcp_rtt_update_buffer().
+ */
+#define LCP_RTT_MAGIC 0x19450425
+#define LCP_RTT_HEADER_LENGTH 4
+#define LCP_RTT_FILE_SIZE 8192
+#define LCP_RTT_ELEMENTS (LCP_RTT_FILE_SIZE / sizeof(u_int32_t) - LCP_RTT_HEADER_LENGTH) / 2
 
 /*
  * LCP-related command-line options.
  */
 int	lcp_echo_interval = 0; 	/* Interval between LCP echo-requests */
 int	lcp_echo_fails = 0;	/* Tolerance to unanswered echo-requests */
+bool	lcp_echo_adaptive = 0;	/* request echo only if the link was idle */
+char	*lcp_rtt_file = NULL;	/* measure the RTT of LCP echo-requests */
 bool	lax_recv = 0;		/* accept control chars in asyncmap */
 bool	noendpoint = 0;		/* don't send/accept endpoint discriminator */
 
-static int noopt __P((char **));
+static int noopt(char **);
 
-#ifdef HAVE_MULTILINK
-static int setendpoint __P((char **));
-static void printendpoint __P((option_t *, void (*)(void *, char *, ...),
-			       void *));
-#endif /* HAVE_MULTILINK */
+#ifdef PPP_WITH_MULTILINK
+static int setendpoint(char **);
+static void printendpoint(option_t *, void (*)(void *, char *, ...), void *);
+#endif /* PPP_WITH_MULTILINK */
 
-static option_t lcp_option_list[] = {
+static struct option lcp_option_list[] = {
     /* LCP options */
     { "-all", o_special_noarg, (void *)noopt,
       "Don't request/allow any LCP options" },
@@ -151,6 +165,11 @@ static option_t lcp_option_list[] = {
       OPT_PRIO },
     { "lcp-echo-interval", o_int, &lcp_echo_interval,
       "Set time in seconds between LCP echo requests", OPT_PRIO },
+    { "lcp-echo-adaptive", o_bool, &lcp_echo_adaptive,
+      "Suppress LCP echo requests if traffic was received", 1 },
+    { "lcp-rtt-file", o_string, &lcp_rtt_file,
+      "Filename for logging the round-trip time of LCP echo requests",
+      OPT_PRIO | OPT_PRIV },
     { "lcp-restart", o_int, &lcp_fsm[0].timeouttime,
       "Set time in seconds between LCP retransmissions", OPT_PRIO },
     { "lcp-max-terminate", o_int, &lcp_fsm[0].maxtermtransmits,
@@ -163,7 +182,7 @@ static option_t lcp_option_list[] = {
     { "receive-all", o_bool, &lax_recv,
       "Accept all received control characters", 1 },
 
-#ifdef HAVE_MULTILINK
+#ifdef PPP_WITH_MULTILINK
     { "mrru", o_int, &lcp_wantoptions[0].mrru,
       "Maximum received packet size for multilink bundle",
       OPT_PRIO, &lcp_wantoptions[0].neg_mrru },
@@ -178,7 +197,7 @@ static option_t lcp_option_list[] = {
     { "endpoint", o_special, (void *) setendpoint,
       "Endpoint discriminator for multilink",
       OPT_PRIO | OPT_A2PRINTER, (void *) printendpoint },
-#endif /* HAVE_MULTILINK */
+#endif /* PPP_WITH_MULTILINK */
 
     { "noendpoint", o_bool, &noendpoint,
       "Don't send or accept multilink endpoint discriminator", 1 },
@@ -196,37 +215,39 @@ lcp_options lcp_hisoptions[NUM_PPP];	/* Options that we ack'd */
 static int lcp_echos_pending = 0;	/* Number of outstanding echo msgs */
 static int lcp_echo_number   = 0;	/* ID number of next echo frame */
 static int lcp_echo_timer_running = 0;  /* set if a timer is running */
+static int lcp_rtt_file_fd = 0;		/* fd for the opened LCP RTT file */
+static u_int32_t *lcp_rtt_buffer = NULL; /* the mmap'ed LCP RTT file */
 
 static u_char nak_buffer[PPP_MRU];	/* where we construct a nak packet */
 
 /*
  * Callbacks for fsm code.  (CI = Configuration Information)
  */
-static void lcp_resetci __P((fsm *));	/* Reset our CI */
-static int  lcp_cilen __P((fsm *));		/* Return length of our CI */
-static void lcp_addci __P((fsm *, u_char *, int *)); /* Add our CI to pkt */
-static int  lcp_ackci __P((fsm *, u_char *, int)); /* Peer ack'd our CI */
-static int  lcp_nakci __P((fsm *, u_char *, int, int)); /* Peer nak'd our CI */
-static int  lcp_rejci __P((fsm *, u_char *, int)); /* Peer rej'd our CI */
-static int  lcp_reqci __P((fsm *, u_char *, int *, int)); /* Rcv peer CI */
-static void lcp_up __P((fsm *));		/* We're UP */
-static void lcp_down __P((fsm *));		/* We're DOWN */
-static void lcp_starting __P((fsm *));	/* We need lower layer up */
-static void lcp_finished __P((fsm *));	/* We need lower layer down */
-static int  lcp_extcode __P((fsm *, int, int, u_char *, int));
-static void lcp_rprotrej __P((fsm *, u_char *, int));
+static void lcp_resetci(fsm *);	/* Reset our CI */
+static int  lcp_cilen(fsm *);		/* Return length of our CI */
+static void lcp_addci(fsm *, u_char *, int *); /* Add our CI to pkt */
+static int  lcp_ackci(fsm *, u_char *, int); /* Peer ack'd our CI */
+static int  lcp_nakci(fsm *, u_char *, int, int); /* Peer nak'd our CI */
+static int  lcp_rejci(fsm *, u_char *, int); /* Peer rej'd our CI */
+static int  lcp_reqci(fsm *, u_char *, int *, int); /* Rcv peer CI */
+static void lcp_up(fsm *);		/* We're UP */
+static void lcp_down(fsm *);		/* We're DOWN */
+static void lcp_starting(fsm *);	/* We need lower layer up */
+static void lcp_finished(fsm *);	/* We need lower layer down */
+static int  lcp_extcode(fsm *, int, int, u_char *, int);
+static void lcp_rprotrej(fsm *, u_char *, int);
 
 /*
  * routines to send LCP echos to peer
  */
 
-static void lcp_echo_lowerup __P((int));
-static void lcp_echo_lowerdown __P((int));
-static void LcpEchoTimeout __P((void *));
-static void lcp_received_echo_reply __P((fsm *, int, u_char *, int));
-static void LcpSendEchoRequest __P((fsm *));
-static void LcpLinkFailure __P((fsm *));
-static void LcpEchoCheck __P((fsm *));
+static void lcp_echo_lowerup(int);
+static void lcp_echo_lowerdown(int);
+static void LcpEchoTimeout(void *);
+static void lcp_received_echo_reply(fsm *, int, u_char *, int);
+static void LcpSendEchoRequest(fsm *);
+static void LcpLinkFailure(fsm *);
+static void LcpEchoCheck(fsm *);
 
 static fsm_callbacks lcp_callbacks = {	/* LCP callback routines */
     lcp_resetci,		/* Reset our Configuration Information */
@@ -251,11 +272,10 @@ static fsm_callbacks lcp_callbacks = {	/* LCP callback routines */
  * Some of these are called directly.
  */
 
-static void lcp_init __P((int));
-static void lcp_input __P((int, u_char *, int));
-static void lcp_protrej __P((int));
-static int  lcp_printpkt __P((u_char *, int,
-			      void (*) __P((void *, char *, ...)), void *));
+static void lcp_init(int);
+static void lcp_input(int, u_char *, int);
+static void lcp_protrej(int);
+static int  lcp_printpkt(u_char *, int, void (*)(void *, char *, ...), void *);
 
 struct protent lcp_protent = {
     PPP_LCP,
@@ -297,8 +317,7 @@ int lcp_loopbackfail = DEFLOOPBACKFAIL;
  * noopt - Disable all options (why?).
  */
 static int
-noopt(argv)
-    char **argv;
+noopt(char **argv)
 {
     BZERO((char *) &lcp_wantoptions[0], sizeof (struct lcp_options));
     BZERO((char *) &lcp_allowoptions[0], sizeof (struct lcp_options));
@@ -306,35 +325,30 @@ noopt(argv)
     return (1);
 }
 
-#ifdef HAVE_MULTILINK
+#ifdef PPP_WITH_MULTILINK
 static int
-setendpoint(argv)
-    char **argv;
+setendpoint(char **argv)
 {
     if (str_to_epdisc(&lcp_wantoptions[0].endpoint, *argv)) {
 	lcp_wantoptions[0].neg_endpoint = 1;
 	return 1;
     }
-    option_error("Can't parse '%s' as an endpoint discriminator", *argv);
+    ppp_option_error("Can't parse '%s' as an endpoint discriminator", *argv);
     return 0;
 }
 
 static void
-printendpoint(opt, printer, arg)
-    option_t *opt;
-    void (*printer) __P((void *, char *, ...));
-    void *arg;
+printendpoint(option_t *opt, void (*printer)(void *, char *, ...), void *arg)
 {
 	printer(arg, "%s", epdisc_to_str(&lcp_wantoptions[0].endpoint));
 }
-#endif /* HAVE_MULTILINK */
+#endif /* PPP_WITH_MULTILINK */
 
 /*
  * lcp_init - Initialize LCP.
  */
 static void
-lcp_init(unit)
-    int unit;
+lcp_init(int unit)
 {
     fsm *f = &lcp_fsm[unit];
     lcp_options *wo = &lcp_wantoptions[unit];
@@ -373,8 +387,7 @@ lcp_init(unit)
  * lcp_open - LCP is allowed to come up.
  */
 void
-lcp_open(unit)
-    int unit;
+lcp_open(int unit)
 {
     fsm *f = &lcp_fsm[unit];
     lcp_options *wo = &lcp_wantoptions[unit];
@@ -392,18 +405,16 @@ lcp_open(unit)
  * lcp_close - Take LCP down.
  */
 void
-lcp_close(unit, reason)
-    int unit;
-    char *reason;
+lcp_close(int unit, char *reason)
 {
     fsm *f = &lcp_fsm[unit];
     int oldstate;
 
-    if (phase != PHASE_DEAD && phase != PHASE_MASTER)
+    if (!in_phase(PHASE_DEAD) && !in_phase(PHASE_MASTER))
 	new_phase(PHASE_TERMINATE);
 
     if (f->flags & DELAYED_UP) {
-	untimeout(lcp_delayed_up, f);
+	UNTIMEOUT(lcp_delayed_up, f);
 	f->state = STOPPED;
     }
     oldstate = f->state;
@@ -427,8 +438,7 @@ lcp_close(unit, reason)
  * lcp_lowerup - The lower layer is up.
  */
 void
-lcp_lowerup(unit)
-    int unit;
+lcp_lowerup(int unit)
 {
     lcp_options *wo = &lcp_wantoptions[unit];
     fsm *f = &lcp_fsm[unit];
@@ -446,7 +456,7 @@ lcp_lowerup(unit)
 
     if (listen_time != 0) {
 	f->flags |= DELAYED_UP;
-	timeout(lcp_delayed_up, f, 0, listen_time * 1000);
+	ppp_timeout(lcp_delayed_up, f, 0, listen_time * 1000);
     } else
 	fsm_lowerup(f);
 }
@@ -456,14 +466,13 @@ lcp_lowerup(unit)
  * lcp_lowerdown - The lower layer is down.
  */
 void
-lcp_lowerdown(unit)
-    int unit;
+lcp_lowerdown(int unit)
 {
     fsm *f = &lcp_fsm[unit];
 
     if (f->flags & DELAYED_UP) {
 	f->flags &= ~DELAYED_UP;
-	untimeout(lcp_delayed_up, f);
+	UNTIMEOUT(lcp_delayed_up, f);
     } else
 	fsm_lowerdown(&lcp_fsm[unit]);
 }
@@ -473,8 +482,7 @@ lcp_lowerdown(unit)
  * lcp_delayed_up - Bring the lower layer up now.
  */
 static void
-lcp_delayed_up(arg)
-    void *arg;
+lcp_delayed_up(void *arg)
 {
     fsm *f = arg;
 
@@ -489,16 +497,13 @@ lcp_delayed_up(arg)
  * lcp_input - Input LCP packet.
  */
 static void
-lcp_input(unit, p, len)
-    int unit;
-    u_char *p;
-    int len;
+lcp_input(int unit, u_char *p, int len)
 {
     fsm *f = &lcp_fsm[unit];
 
     if (f->flags & DELAYED_UP) {
 	f->flags &= ~DELAYED_UP;
-	untimeout(lcp_delayed_up, f);
+	UNTIMEOUT(lcp_delayed_up, f);
 	fsm_lowerup(f);
     }
     fsm_input(f, p, len);
@@ -508,11 +513,7 @@ lcp_input(unit, p, len)
  * lcp_extcode - Handle a LCP-specific code.
  */
 static int
-lcp_extcode(f, code, id, inp, len)
-    fsm *f;
-    int code, id;
-    u_char *inp;
-    int len;
+lcp_extcode(fsm *f, int code, int id, u_char *inp, int len)
 {
     u_char *magp;
 
@@ -551,10 +552,7 @@ lcp_extcode(f, code, id, inp, len)
  * Figure out which protocol is rejected and inform it.
  */
 static void
-lcp_rprotrej(f, inp, len)
-    fsm *f;
-    u_char *inp;
-    int len;
+lcp_rprotrej(fsm *f, u_char *inp, int len)
 {
     int i;
     struct protent *protp;
@@ -606,8 +604,7 @@ lcp_rprotrej(f, inp, len)
  */
 /*ARGSUSED*/
 static void
-lcp_protrej(unit)
-    int unit;
+lcp_protrej(int unit)
 {
     /*
      * Can't reject LCP!
@@ -621,10 +618,7 @@ lcp_protrej(unit)
  * lcp_sprotrej - Send a Protocol-Reject for some protocol.
  */
 void
-lcp_sprotrej(unit, p, len)
-    int unit;
-    u_char *p;
-    int len;
+lcp_sprotrej(int unit, u_char *p, int len)
 {
     /*
      * Send back the protocol and the information field of the
@@ -642,8 +636,7 @@ lcp_sprotrej(unit, p, len)
  * lcp_resetci - Reset our CI.
  */
 static void
-lcp_resetci(f)
-    fsm *f;
+lcp_resetci(fsm *f)
 {
     lcp_options *wo = &lcp_wantoptions[f->unit];
     lcp_options *go = &lcp_gotoptions[f->unit];
@@ -668,8 +661,7 @@ lcp_resetci(f)
  * lcp_cilen - Return length of our CI.
  */
 static int
-lcp_cilen(f)
-    fsm *f;
+lcp_cilen(fsm *f)
 {
     lcp_options *go = &lcp_gotoptions[f->unit];
 
@@ -704,10 +696,7 @@ lcp_cilen(f)
  * lcp_addci - Add our desired CIs to a packet.
  */
 static void
-lcp_addci(f, ucp, lenp)
-    fsm *f;
-    u_char *ucp;
-    int *lenp;
+lcp_addci(fsm *f, u_char *ucp, int *lenp)
 {
     lcp_options *go = &lcp_gotoptions[f->unit];
     u_char *start_ucp = ucp;
@@ -792,10 +781,7 @@ lcp_addci(f, ucp, lenp)
  *	1 - Ack was good.
  */
 static int
-lcp_ackci(f, p, len)
-    fsm *f;
-    u_char *p;
-    int len;
+lcp_ackci(fsm *f, u_char *p, int len)
 {
     lcp_options *go = &lcp_gotoptions[f->unit];
     u_char cilen, citype, cichar;
@@ -947,11 +933,7 @@ bad:
  *	1 - Nak was good.
  */
 static int
-lcp_nakci(f, p, len, treat_as_reject)
-    fsm *f;
-    u_char *p;
-    int len;
-    int treat_as_reject;
+lcp_nakci(fsm *f, u_char *p, int len, int treat_as_reject)
 {
     lcp_options *go = &lcp_gotoptions[f->unit];
     lcp_options *wo = &lcp_wantoptions[f->unit];
@@ -1315,7 +1297,7 @@ lcp_nakci(f, p, len, treat_as_reject)
 	if (looped_back) {
 	    if (++try.numloops >= lcp_loopbackfail) {
 		notice("Serial line is looped back.");
-		status = EXIT_LOOPBACK;
+		ppp_set_status(EXIT_LOOPBACK);
 		lcp_close(f->unit, "Loopback detected");
 	    }
 	} else
@@ -1341,10 +1323,7 @@ bad:
  *	1 - Reject was good.
  */
 static int
-lcp_rejci(f, p, len)
-    fsm *f;
-    u_char *p;
-    int len;
+lcp_rejci(fsm *f, u_char *p, int len)
 {
     lcp_options *go = &lcp_gotoptions[f->unit];
     u_char cichar;
@@ -1500,11 +1479,7 @@ bad:
  * CONFNAK; returns CONFREJ if it can't return CONFACK.
  */
 static int
-lcp_reqci(f, inp, lenp, reject_if_disagree)
-    fsm *f;
-    u_char *inp;		/* Requested CIs */
-    int *lenp;			/* Length of requested CIs */
-    int reject_if_disagree;
+lcp_reqci(fsm *f, u_char *inp, int *lenp, int reject_if_disagree)
 {
     lcp_options *go = &lcp_gotoptions[f->unit];
     lcp_options *ho = &lcp_hisoptions[f->unit];
@@ -1891,8 +1866,7 @@ endswitch:
  * lcp_up - LCP has come UP.
  */
 static void
-lcp_up(f)
-    fsm *f;
+lcp_up(fsm *f)
 {
     lcp_options *wo = &lcp_wantoptions[f->unit];
     lcp_options *ho = &lcp_hisoptions[f->unit];
@@ -1916,10 +1890,10 @@ lcp_up(f)
      */
     mtu = ho->neg_mru? ho->mru: PPP_MRU;
     mru = go->neg_mru? MAX(wo->mru, go->mru): PPP_MRU;
-#ifdef HAVE_MULTILINK
+#ifdef PPP_WITH_MULTILINK
     if (!(multilink && go->neg_mrru && ho->neg_mrru))
-#endif /* HAVE_MULTILINK */
-	netif_set_mtu(f->unit, MIN(MIN(mtu, mru), ao->mru));
+#endif /* PPP_WITH_MULTILINK */
+	ppp_set_mtu(f->unit, MIN(MIN(mtu, mru), ao->mru));
     ppp_send_config(f->unit, mtu,
 		    (ho->neg_asyncmap? ho->asyncmap: 0xffffffff),
 		    ho->neg_pcompression, ho->neg_accompression);
@@ -1942,8 +1916,7 @@ lcp_up(f)
  * Alert other protocols.
  */
 static void
-lcp_down(f)
-    fsm *f;
+lcp_down(fsm *f)
 {
     lcp_options *go = &lcp_gotoptions[f->unit];
 
@@ -1963,8 +1936,7 @@ lcp_down(f)
  * lcp_starting - LCP needs the lower layer up.
  */
 static void
-lcp_starting(f)
-    fsm *f;
+lcp_starting(fsm *f)
 {
     link_required(f->unit);
 }
@@ -1974,8 +1946,7 @@ lcp_starting(f)
  * lcp_finished - LCP has finished with the lower layer.
  */
 static void
-lcp_finished(f)
-    fsm *f;
+lcp_finished(fsm *f)
 {
     link_terminated(f->unit);
 }
@@ -1992,11 +1963,7 @@ static char *lcp_codenames[] = {
 };
 
 static int
-lcp_printpkt(p, plen, printer, arg)
-    u_char *p;
-    int plen;
-    void (*printer) __P((void *, char *, ...));
-    void *arg;
+lcp_printpkt(u_char *p, int plen, void (*printer)(void *, char *, ...), void *arg)
 {
     int code, id, len, olen, i;
     u_char *pstart, *optend;
@@ -2147,7 +2114,7 @@ lcp_printpkt(p, plen, printer, arg)
 		}
 		break;
 	    case CI_EPDISC:
-#ifdef HAVE_MULTILINK
+#ifdef PPP_WITH_MULTILINK
 		if (olen >= CILEN_CHAR) {
 		    struct epdisc epd;
 		    p += 2;
@@ -2235,13 +2202,12 @@ lcp_printpkt(p, plen, printer, arg)
  */
 
 static
-void LcpLinkFailure (f)
-    fsm *f;
+void LcpLinkFailure (fsm *f)
 {
     if (f->state == OPENED) {
 	info("No response to %d echo-requests", lcp_echos_pending);
         notice("Serial link appears to be disconnected.");
-	status = EXIT_PEER_DEAD;
+	ppp_set_status(EXIT_PEER_DEAD);
 	lcp_close(f->unit, "Peer not responding");
     }
 }
@@ -2251,8 +2217,7 @@ void LcpLinkFailure (f)
  */
 
 static void
-LcpEchoCheck (f)
-    fsm *f;
+LcpEchoCheck (fsm *f)
 {
     LcpSendEchoRequest (f);
     if (f->state != OPENED)
@@ -2272,8 +2237,7 @@ LcpEchoCheck (f)
  */
 
 static void
-LcpEchoTimeout (arg)
-    void *arg;
+LcpEchoTimeout (void *arg)
 {
     if (lcp_echo_timer_running != 0) {
         lcp_echo_timer_running = 0;
@@ -2281,16 +2245,79 @@ LcpEchoTimeout (arg)
     }
 }
 
+/*
+ * Log the round-trip time (RTT) of the received LCP echo-request.
+ *
+ * The header section at the beginning of lcp_rtt_file contains
+ * LCP_RTT_HEADER_LENGTH fields, each a u_int32_t in network byte order:
+ * [0] LCP_RTT_MAGIC
+ * [1] status (1: the file is open and is being written)
+ * [2] index of the most recently updated element
+ * [3] the value of the lcp-echo-interval parameter
+ *
+ * The header is followed by a ring buffer of LCP_RTT_ELEMENTS elements, each
+ * containing a pair of u_int32_t in network byte order with this content:
+ * [0] UNIX timestamp
+ * [1] bits 24-31: the number of lost LCP echo replies
+ *     bits 0-23:  the measured RTT in microseconds
+ *
+ * The timestamp is unsigned to support storing dates beyond 2038.
+ *
+ * Consumers of lcp_rtt_file are expected to:
+ * - read the complete file of arbitrary length
+ * - check the magic number
+ * - process the data elements starting at the index
+ * - ignore any elements with a timestamp of 0
+ */
+static void
+lcp_rtt_update_buffer (unsigned long rtt)
+{
+    volatile u_int32_t *const ring_header = lcp_rtt_buffer;
+    volatile u_int32_t *const ring_buffer = lcp_rtt_buffer
+	+ LCP_RTT_HEADER_LENGTH;
+    unsigned int next_entry, lost;
+
+    /* choose the next entry where the data will be stored */
+    if (ntohl(ring_header[2]) >= (LCP_RTT_ELEMENTS - 1) * 2)
+	next_entry = 0;				/* go back to the beginning */
+    else
+	next_entry = ntohl(ring_header[2]) + 2;	/* use the next one */
+
+    /* update the data element */
+    /* storing the timestamp in an *unsigned* long allows dates up to 2106 */
+    ring_buffer[next_entry] = htonl((u_int32_t) time(NULL));
+    lost = lcp_echos_pending - 1;
+    if (lost > 0xFF)
+	lost = 0xFF;		/* truncate the lost packets count to 256 */
+    if (rtt > 0xFFFFFF)
+	rtt = 0xFFFFFF;		/* truncate the RTT to 16777216 */
+    /* use bits 24-31 for the lost packets count and bits 0-23 for the RTT */
+    ring_buffer[next_entry + 1] = htonl((u_int32_t) ((lost << 24) + rtt));
+
+    /* update the pointer to the (just updated) most current data element */
+    ring_header[2] = htonl(next_entry);
+
+    /* In theory, CPUs implementing a weakly-consistent memory model do not
+     * guarantee that these three memory store operations to the buffer will
+     * be seen in the same order by the reader process.
+     * This means that a process reading the file could see the index
+     * having been updated before the element that the index points to had
+     * been written.
+     * But in practice we expect that the read(2) system call used by
+     * consumers processes is atomic with respect to the following msync(2)
+     * call, so we ignore the issue.
+     */
+
+    if (msync(lcp_rtt_buffer, LCP_RTT_FILE_SIZE, MS_ASYNC) < 0)
+	error("msync() for %s failed: %m", lcp_rtt_file);
+}
+
 /*
  * LcpEchoReply - LCP has received a reply to the echo
  */
 
 static void
-lcp_received_echo_reply (f, id, inp, len)
-    fsm *f;
-    int id;
-    u_char *inp;
-    int len;
+lcp_received_echo_reply (fsm *f, int id, u_char *inp, int len)
 {
     u_int32_t magic;
 
@@ -2306,6 +2333,30 @@ lcp_received_echo_reply (f, id, inp, len)
 	return;
     }
 
+    if (lcp_rtt_file_fd && len >= 16) {
+	long lcp_rtt_magic;
+
+	/*
+	 * If the magic word is found at the beginning of the data section
+	 * of the frame then read the timestamp which follows and subtract
+	 * it from the current time to compute the round trip time.
+	 */
+	GETLONG(lcp_rtt_magic, inp);
+	if (lcp_rtt_magic == LCP_RTT_MAGIC) {
+	    struct timespec ts;
+	    unsigned long req_sec, req_nsec, rtt;
+
+	    clock_gettime(CLOCK_MONOTONIC, &ts);
+	    GETLONG(req_sec, inp);
+	    GETLONG(req_nsec, inp);
+	    /* compute the RTT in microseconds */
+	    rtt = (ts.tv_sec - req_sec) * 1000000
+		+ (ts.tv_nsec / 1000 - req_nsec / 1000);
+	    /* log the RTT */
+	    lcp_rtt_update_buffer(rtt);
+	}
+    }
+
     /* Reset the number of outstanding echo frames */
     lcp_echos_pending = 0;
 }
@@ -2315,11 +2366,10 @@ lcp_received_echo_reply (f, id, inp, len)
  */
 
 static void
-LcpSendEchoRequest (f)
-    fsm *f;
+LcpSendEchoRequest (fsm *f)
 {
     u_int32_t lcp_magic;
-    u_char pkt[4], *pktp;
+    u_char pkt[16], *pktp;
 
     /*
      * Detect the failure of the peer at this point.
@@ -2331,6 +2381,22 @@ LcpSendEchoRequest (f)
 	}
     }
 
+    /*
+     * If adaptive echos have been enabled, only send the echo request if
+     * no traffic was received since the last one.
+     */
+    if (lcp_echo_adaptive) {
+	static unsigned int last_pkts_in = 0;
+	struct pppd_stats cur_stats;
+
+	if (get_ppp_stats(f->unit, &cur_stats) && cur_stats.pkts_in != last_pkts_in) {
+	    last_pkts_in = cur_stats.pkts_in;
+	    /* receipt of traffic indicates the link is working... */
+	    lcp_echos_pending = 0;
+	    return;
+	}
+    }
+
     /*
      * Make and send the echo request frame.
      */
@@ -2338,18 +2404,79 @@ LcpSendEchoRequest (f)
         lcp_magic = lcp_gotoptions[f->unit].magicnumber;
 	pktp = pkt;
 	PUTLONG(lcp_magic, pktp);
+
+	/* Put a timestamp in the data section of the frame */
+	if (lcp_rtt_file_fd) {
+	    struct timespec ts;
+
+	    PUTLONG(LCP_RTT_MAGIC, pktp);
+	    clock_gettime(CLOCK_MONOTONIC, &ts);
+	    PUTLONG((u_int32_t)ts.tv_sec, pktp);
+	    PUTLONG((u_int32_t)ts.tv_nsec, pktp);
+	}
+
         fsm_sdata(f, ECHOREQ, lcp_echo_number++ & 0xFF, pkt, pktp - pkt);
 	++lcp_echos_pending;
     }
 }
 
+static void
+lcp_rtt_open_file (void)
+{
+    volatile u_int32_t *ring_header;
+
+    if (!lcp_rtt_file)
+	return;
+
+    lcp_rtt_file_fd = open(lcp_rtt_file, O_RDWR | O_CREAT, 0644);
+    if (lcp_rtt_file_fd < 0) {
+	error("Can't open the RTT log file %s: %m", lcp_rtt_file);
+	lcp_rtt_file_fd = 0;
+	return;
+    }
+
+    if (ftruncate(lcp_rtt_file_fd, LCP_RTT_FILE_SIZE) < 0)
+	fatal("ftruncate() of %s failed: %m", lcp_rtt_file);
+    lcp_rtt_buffer = mmap(0, LCP_RTT_FILE_SIZE, PROT_READ | PROT_WRITE,
+	    MAP_SHARED, lcp_rtt_file_fd, 0);
+    if (lcp_rtt_buffer == MAP_FAILED)
+	fatal("mmap() of %s failed: %m", lcp_rtt_file);
+    ring_header = lcp_rtt_buffer;
+
+    /* initialize the ring buffer */
+    if (ring_header[0] != htonl(LCP_RTT_MAGIC)) {
+	memset(lcp_rtt_buffer, 0, LCP_RTT_FILE_SIZE);
+	ring_header[0] = htonl(LCP_RTT_MAGIC);
+    }
+
+    ring_header[3] = htonl(lcp_echo_interval);
+    ring_header[1] = htonl(1); /* status: LCP up, file opened */
+}
+
+static void
+lcp_rtt_close_file (void)
+{
+    volatile u_int32_t *const ring_header = lcp_rtt_buffer;
+
+    if (!lcp_rtt_file_fd)
+	return;
+
+    ring_header[1] = htonl(0); /* status: LCP down, file closed */
+
+    if (munmap(lcp_rtt_buffer, LCP_RTT_FILE_SIZE) < 0)
+	error("munmap() of %s failed: %m", lcp_rtt_file);
+    if (close(lcp_rtt_file_fd) < 0)
+	error("close() of %s failed: %m", lcp_rtt_file);
+    lcp_rtt_buffer = NULL;
+    lcp_rtt_file_fd = 0;
+}
+
 /*
  * lcp_echo_lowerup - Start the timer for the LCP frame
  */
 
 static void
-lcp_echo_lowerup (unit)
-    int unit;
+lcp_echo_lowerup (int unit)
 {
     fsm *f = &lcp_fsm[unit];
 
@@ -2357,6 +2484,9 @@ lcp_echo_lowerup (unit)
     lcp_echos_pending      = 0;
     lcp_echo_number        = 0;
     lcp_echo_timer_running = 0;
+
+    /* Open the file where the LCP RTT data will be logged */
+    lcp_rtt_open_file();
   
     /* If a timeout interval is specified then start the timer */
     if (lcp_echo_interval != 0)
@@ -2368,8 +2498,7 @@ lcp_echo_lowerup (unit)
  */
 
 static void
-lcp_echo_lowerdown (unit)
-    int unit;
+lcp_echo_lowerdown (int unit)
 {
     fsm *f = &lcp_fsm[unit];
 
@@ -2377,4 +2506,7 @@ lcp_echo_lowerdown (unit)
         UNTIMEOUT (LcpEchoTimeout, f);
         lcp_echo_timer_running = 0;
     }
+
+    /* Close the file containing the LCP RTT data */
+    lcp_rtt_close_file();
 }
