@@ -39,7 +39,7 @@
  * AN ACTION OF CONTRACT, NEGLIGENCE OR OTHER TORTIOUS ACTION, ARISING
  * OUT OF OR IN CONNECTION WITH THE USE OR PERFORMANCE OF THIS SOFTWARE.
  *
- * Copyright (c) 1999-2020 Paul Mackerras. All rights reserved.
+ * Copyright (c) 1999-2024 Paul Mackerras. All rights reserved.
  *
  * Redistribution and use in source and binary forms, with or without
  * modification, are permitted provided that the following conditions
@@ -48,14 +48,10 @@
  * 1. Redistributions of source code must retain the above copyright
  *    notice, this list of conditions and the following disclaimer.
  *
- * 2. The name(s) of the authors of this software must not be used to
- *    endorse or promote products derived from this software without
- *    prior written permission.
- *
- * 3. Redistributions of any form whatsoever must retain the following
- *    acknowledgment:
- *    "This product includes software developed by Paul Mackerras
- *     <paulus@ozlabs.org>".
+ * 2. Redistributions in binary form must reproduce the above copyright
+ *    notice, this list of conditions and the following disclaimer in
+ *    the documentation and/or other materials provided with the
+ *    distribution.
  *
  * THE AUTHORS OF THIS SOFTWARE DISCLAIM ALL WARRANTIES WITH REGARD TO
  * THIS SOFTWARE, INCLUDING ALL IMPLIED WARRANTIES OF MERCHANTABILITY
@@ -611,8 +607,10 @@ main(int argc, char *argv[])
 	while (phase != PHASE_DEAD) {
 	    handle_events();
 	    get_input();
-	    if (kill_link)
+	    if (kill_link) {
 		lcp_close(0, "User request");
+		need_holdoff = 0;
+	    }
 	    if (asked_to_quit) {
 		bundle_terminating = 1;
 		if (phase == PHASE_MASTER)
@@ -852,6 +850,8 @@ set_ifunit(int iskey)
 	create_pidfile(getpid());	/* write pid to file */
 	create_linkpidfile(getpid());
     }
+    if (*remote_number)
+        ppp_script_setenv("REMOTENUMBER", remote_number, 0);
     run_net_script(path_net_init, 1);
 }
 
@@ -921,6 +921,7 @@ create_pidfile(int pid)
 {
     FILE *pidfile;
 
+    mkdir_recursive(PPP_PATH_VARRUN);
     slprintf(pidfilename, sizeof(pidfilename), "%s/%s.pid",
 	     PPP_PATH_VARRUN, ifname);
     if ((pidfile = fopen(pidfilename, "w")) != NULL) {
@@ -1150,6 +1151,7 @@ get_input(void)
 	notice("Modem hangup");
 	hungup = 1;
 	code = EXIT_HANGUP;
+	need_holdoff = 0;
 	lcp_lowerdown(0);	/* serial link is no longer available */
 	link_terminated(0);
 	return;
@@ -1330,7 +1332,7 @@ print_link_stats(void)
     if (link_stats_print && link_stats_valid) {
        int t = (link_connect_time + 5) / 6;    /* 1/10ths of minutes */
        info("Connect time %d.%d minutes.", t/10, t%10);
-       info("Sent %u bytes, received %u bytes.",
+       info("Sent %llu bytes, received %llu bytes.",
 	    link_stats.bytes_out, link_stats.bytes_in);
        link_stats_print = 0;
     }
